# -*- coding: utf-8 -*-
######################################################
#
#  File Manager
#   Copyright (c) 2024 btk
#   mail@btk.tokyo.jp
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to use
# the Software with or without modification, subject to the following conditions:
# 
# 1. The code may be modified and used freely.
# 2. If you modify the code, please notify the original copyright holder
#    (mail@btk.tokyo.jp) of the modifications made, if possible.
# 3. Redistribution of the original or modified code is prohibited.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# 本ソフトウェアおよび関連文書ファイル（以下「ソフトウェア」）のコピーを
# 入手したすべての人に対し、以下の条件に従うことを条件に、使用および改変の
# 許可を無償で付与します。
# 
# 1. 本コードは自由に改変および使用してかまいません。
# 2. 可能であれば、改変した場合は改変の内容を元の著作権者（mail@btk.tokyo.jp）に連絡してください。
#　　（どういった個所に不満があったのか等を知りたいためなので義務ではありません）
# 3. 元のコードおよび改変されたコードの再配布は禁止します。
# 
# 本ソフトウェアは「現状のまま」提供されており、商品性、特定の目的への適合性、
# および権利非侵害についての保証を含め、明示的または黙示的な保証はありません。
# 著者または著作権者は、契約行為、不法行為、またはその他の行為に関係なく、
# 本ソフトウェアまたはその使用またはその他の取り扱いから生じるいかなる
# 請求、損害、その他の責任についても責任を負いません。
#
######################################################

import os
import glob
import json
import re
import datetime
import subprocess
import math
import threading

from functools import partial

try:
    from PySide2 import QtCore, QtGui, QtWidgets
    from PySide2.QtCore import *
    from PySide2.QtGui import *
    from PySide2.QtWidgets import *
    import shiboken2 as shiboken
except:
    from PySide6 import QtCore, QtGui, QtWidgets
    from PySide6.QtCore import *
    from PySide6.QtGui import *
    from PySide6.QtWidgets import *
    import shiboken6 as shiboken

import maya.OpenMaya as OpenMaya
import maya.OpenMayaUI as OpenMayaUI
from maya.app.general.mayaMixin import MayaQWidgetDockableMixin
import maya.cmds as cmds
import maya.mel as mel

from . import btk_slide_vari_setter
import importlib
importlib.reload(btk_slide_vari_setter)

from .btk_slide_vari_setter import BTKLineEditSlideVari, BTKSpinBoxEditSlideVari, BTKDoubleSpinBoxEditSlideVari

root_dir = os.path.dirname(__file__)
icon_dir = os.path.join(root_dir,'ui/icon').replace('\\','/')
user_settingfile = 'btk_filemanager'
homedir = os.getenv('USERPROFILE')
ini_dir = os.path.join(homedir,'.btk')
user_config_file = os.path.join(ini_dir,'%s.json'%user_settingfile)

def convert_size(size):
    units = ('B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB')
    i = math.floor(math.log(size, 1024)) if size > 0 else 0
    size = round(size / 1024 ** i, 2)

    return f'{size} {units[i]}'

class Ui_btkFileManagerOptionDialogg(object):
    def setupUi(self, Dialog):
        if not Dialog.objectName():
            Dialog.setObjectName(u"Dialog")
        Dialog.resize(702, 379)
        self.verticalLayout = QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.line = QFrame(Dialog)
        self.line.setObjectName(u"line")
        self.line.setFrameShape(QFrame.HLine)
        self.line.setFrameShadow(QFrame.Sunken)

        self.gridLayout.addWidget(self.line, 5, 0, 1, 2)

        self.search = QLineEdit(Dialog)
        self.search.setObjectName(u"search")

        self.gridLayout.addWidget(self.search, 6, 1, 1, 1)

        self.recursive = QCheckBox(Dialog)
        self.recursive.setObjectName(u"recursive")

        self.gridLayout.addWidget(self.recursive, 4, 1, 1, 1)

        self.label_2 = QLabel(Dialog)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout.addWidget(self.label_2, 3, 0, 1, 1)

        self.filter = QComboBox(Dialog)
        self.filter.addItem("")
        self.filter.addItem("")
        self.filter.addItem("")
        self.filter.addItem("")
        self.filter.setObjectName(u"filter")
        self.filter.setMinimumSize(QSize(0, 0))

        self.gridLayout.addWidget(self.filter, 8, 1, 1, 1)

        self.label = QLabel(Dialog)
        self.label.setObjectName(u"label")
        self.label.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)

        self.name = QLineEdit(Dialog)
        self.name.setObjectName(u"name")

        self.gridLayout.addWidget(self.name, 0, 1, 1, 1)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_3.addItem(self.horizontalSpacer_2)

        self.apply_doubleclick = QCheckBox(Dialog)
        self.apply_doubleclick.setObjectName(u"apply_doubleclick")

        self.horizontalLayout_3.addWidget(self.apply_doubleclick)


        self.gridLayout.addLayout(self.horizontalLayout_3, 11, 0, 1, 1)

        self.gridLayout_2 = QGridLayout()
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.dc_none = QRadioButton(Dialog)
        self.dc_none.setObjectName(u"dc_none")
        self.dc_none.setChecked(True)

        self.gridLayout_2.addWidget(self.dc_none, 0, 0, 1, 1)

        self.dc_open = QRadioButton(Dialog)
        self.dc_open.setObjectName(u"dc_open")

        self.gridLayout_2.addWidget(self.dc_open, 0, 1, 1, 2)

        self.dc_import = QRadioButton(Dialog)
        self.dc_import.setObjectName(u"dc_import")

        self.gridLayout_2.addWidget(self.dc_import, 0, 3, 1, 1)

        self.dc_reference = QRadioButton(Dialog)
        self.dc_reference.setObjectName(u"dc_reference")

        self.gridLayout_2.addWidget(self.dc_reference, 1, 0, 1, 2)

        self.dc_gpucache = QRadioButton(Dialog)
        self.dc_gpucache.setObjectName(u"dc_gpucache")

        self.gridLayout_2.addWidget(self.dc_gpucache, 1, 2, 1, 2)


        self.gridLayout.addLayout(self.gridLayout_2, 11, 1, 1, 1)

        self.description = QLineEdit(Dialog)
        self.description.setObjectName(u"description")

        self.gridLayout.addWidget(self.description, 2, 1, 1, 1)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer)

        self.apply_filter = QCheckBox(Dialog)
        self.apply_filter.setObjectName(u"apply_filter")

        self.horizontalLayout_2.addWidget(self.apply_filter)


        self.gridLayout.addLayout(self.horizontalLayout_2, 6, 0, 1, 1)

        self.label_3 = QLabel(Dialog)
        self.label_3.setObjectName(u"label_3")
        self.label_3.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)

        self.line_2 = QFrame(Dialog)
        self.line_2.setObjectName(u"line_2")
        self.line_2.setFrameShape(QFrame.HLine)
        self.line_2.setFrameShadow(QFrame.Sunken)

        self.gridLayout.addWidget(self.line_2, 10, 0, 1, 2)

        self.exclusion = QLineEdit(Dialog)
        self.exclusion.setObjectName(u"exclusion")

        self.gridLayout.addWidget(self.exclusion, 7, 1, 1, 1)

        self.latest = QCheckBox(Dialog)
        self.latest.setObjectName(u"latest")

        self.gridLayout.addWidget(self.latest, 9, 1, 1, 1)

        self.label_4 = QLabel(Dialog)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setAlignment(Qt.AlignRight|Qt.AlignTrailing|Qt.AlignVCenter)

        self.gridLayout.addWidget(self.label_4, 1, 0, 1, 1)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setSpacing(2)
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.default_icons = QComboBox(Dialog)
        self.default_icons.setObjectName(u"default_icons")

        self.horizontalLayout_4.addWidget(self.default_icons)

        self.icon_file = QLineEdit(Dialog)
        self.icon_file.setObjectName(u"icon_file")

        self.horizontalLayout_4.addWidget(self.icon_file)

        self.set_icon = QPushButton(Dialog)
        self.set_icon.setObjectName(u"set_icon")
        self.set_icon.setMaximumSize(QSize(25, 16777215))

        self.horizontalLayout_4.addWidget(self.set_icon)

        self.icon = QLabel(Dialog)
        self.icon.setObjectName(u"icon")
        self.icon.setMinimumSize(QSize(32, 0))
        self.icon.setMaximumSize(QSize(32, 32))

        self.horizontalLayout_4.addWidget(self.icon)


        self.gridLayout.addLayout(self.horizontalLayout_4, 1, 1, 1, 1)

        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setSpacing(1)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(2)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.location = QLineEdit(Dialog)
        self.location.setObjectName(u"location")

        self.horizontalLayout.addWidget(self.location)

        self.set_location = QPushButton(Dialog)
        self.set_location.setObjectName(u"set_location")
        self.set_location.setMaximumSize(QSize(25, 16777215))

        self.horizontalLayout.addWidget(self.set_location)


        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.label_5 = QLabel(Dialog)
        self.label_5.setObjectName(u"label_5")

        self.horizontalLayout_5.addWidget(self.label_5)

        self.insert_text1 = QPushButton(Dialog)
        self.insert_text1.setObjectName(u"insert_text1")

        self.horizontalLayout_5.addWidget(self.insert_text1)

        self.horizontalSpacer_3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_5.addItem(self.horizontalSpacer_3)


        self.verticalLayout_2.addLayout(self.horizontalLayout_5)


        self.gridLayout.addLayout(self.verticalLayout_2, 3, 1, 1, 1)

        self.gridLayout.setColumnStretch(1, 1)

        self.verticalLayout.addLayout(self.gridLayout)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)

        self.buttonBox = QDialogButtonBox(Dialog)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)

        self.verticalLayout.addWidget(self.buttonBox)


        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)

        QMetaObject.connectSlotsByName(Dialog)
    # setupUi

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QCoreApplication.translate("Dialog", u"Dialog", None))
        self.search.setPlaceholderText(QCoreApplication.translate("Dialog", u"Keyword", None))
        self.recursive.setText(QCoreApplication.translate("Dialog", u"Recursive", None))
        self.label_2.setText(QCoreApplication.translate("Dialog", u"Location", None))
        self.filter.setItemText(0, QCoreApplication.translate("Dialog", u"ma, mb, abc", None))
        self.filter.setItemText(1, QCoreApplication.translate("Dialog", u"ma, mb", None))
        self.filter.setItemText(2, QCoreApplication.translate("Dialog", u"abc", None))
        self.filter.setItemText(3, QCoreApplication.translate("Dialog", u"*", None))

        self.label.setText(QCoreApplication.translate("Dialog", u"Display name", None))
        self.apply_doubleclick.setText(QCoreApplication.translate("Dialog", u"DoubleClick", None))
        self.dc_none.setText(QCoreApplication.translate("Dialog", u"None", None))
        self.dc_open.setText(QCoreApplication.translate("Dialog", u"Open", None))
        self.dc_import.setText(QCoreApplication.translate("Dialog", u"Import", None))
        self.dc_reference.setText(QCoreApplication.translate("Dialog", u"Reference", None))
        self.dc_gpucache.setText(QCoreApplication.translate("Dialog", u"GPUCache", None))
        self.apply_filter.setText(QCoreApplication.translate("Dialog", u"Filter", None))
        self.label_3.setText(QCoreApplication.translate("Dialog", u"Description", None))
        self.exclusion.setPlaceholderText(QCoreApplication.translate("Dialog", u"Exclusion", None))
        self.latest.setText(QCoreApplication.translate("Dialog", u"Latest version only", None))
        self.label_4.setText(QCoreApplication.translate("Dialog", u"Icon", None))
        self.set_icon.setText(QCoreApplication.translate("Dialog", u"..", None))
        self.icon.setText("")
        self.set_location.setText(QCoreApplication.translate("Dialog", u"..", None))
        self.label_5.setText(QCoreApplication.translate("Dialog", u"Insert:", None))
        self.insert_text1.setText(QCoreApplication.translate("Dialog", u"$PROJ", None))
    # retranslateUi

class btkFileManagerOptionDialog(QDialog):
    titleName = 'Option'

    def __init__(self, parent=None):
        super(btkFileManagerOptionDialog, self).__init__(parent)
        self.setObjectName(self.titleName)
        self.ui=Ui_btkFileManagerOptionDialogg()
        wd = os.getcwd()                    #作業ディレクトリをいったん保存
        os.chdir(os.path.dirname(__file__)+'/ui') #作業ディレクトリをPythonファイルの場所に指定　※アイコンなどは作業ディレクトリからの相対パスで読み込まれるため
        self.ui.setupUi(self)               #相対パスを使ったアイコンなどの設定がされる
        os.chdir(wd)                        #作業ディレクトリを元に戻しておく
        self.setWindowTitle(self.titleName)


        default_icons = [os.path.splitext(os.path.basename(x))[0] for x in glob.glob(f'{icon_dir}/file/*.png')]
        default_icons.insert(0,'')
        self.ui.default_icons.addItems(default_icons)
        self.ui.default_icons.currentTextChanged.connect(self.combo_changed)
        self.ui.icon_file.textChanged.connect(self.icon_changed)
        self.dc_actions = QButtonGroup(self)
        self.dc_actions.addButton(self.ui.dc_none,0)
        self.dc_actions.addButton(self.ui.dc_open,1)
        self.dc_actions.addButton(self.ui.dc_import,2)
        self.dc_actions.addButton(self.ui.dc_reference,3)
        self.dc_actions.addButton(self.ui.dc_gpucache,4)

        self.ui.set_icon.clicked.connect(self.set_icon)
        self.ui.set_location.clicked.connect(self.set_location)
        self.ui.insert_text1.clicked.connect(self.insert_text)

    def set_location(self):
        cur = self.ui.location.text()
        path = QFileDialog.getExistingDirectory(self,'Select Direvtory',cur)
        if path:
            self.ui.location.setText(path.replace('\\','/'))

    def insert_text(self):
        _insert_text = self.sender().text()
        
        cur_location = self.ui.location.text()
        pos = self.ui.location.cursorPosition()

        new_text = cur_location[:pos] + _insert_text + cur_location[pos:]
        self.ui.location.setText(new_text)
        self.ui.location.setCursorPosition(pos+len(_insert_text))

    def combo_changed(self):
        text = self.ui.default_icons.currentText()
        if text != '':
            self.ui.icon_file.setText(f'{icon_dir}/file/{text}.png')

    def set_icon(self):
        cur = self.ui.icon_file.text()
        path = QFileDialog.getOpenFileName(self,'Select file',cur)
        if len(path[0])>0:
            self.ui.icon_file.setText(path[0].replace('\\','/'))

    def set_default(self,bookmark):
        if bookmark is None:
            return
        
        self.ui.name.setText(bookmark['name'])
        self.ui.description.setText(bookmark.get('description',''))
        self.ui.icon_file.setText(bookmark.get('icon',''))
        self.ui.location.setText(bookmark['directory'])
        self.ui.recursive.setChecked(bookmark['option'].get('recursive',False))

        self.ui.apply_filter.setChecked(bookmark['option'].get('apply_filter',False))
        self.ui.search.setText(bookmark['option'].get('filter_word',''))
        self.ui.exclusion.setText(bookmark['option'].get('filter_exclusion',''))
        self.ui.filter.setCurrentText(bookmark['option'].get('filter_ext','ma, mb, abc'))
        self.ui.latest.setChecked(bookmark['option'].get('latest',False))

        self.ui.apply_doubleclick.setChecked(bookmark['option'].get('apply_doubleclick',False))
        self.dc_actions.button(bookmark['option'].get('dc_action',0)).setChecked(True)

    def icon_changed(self):
        icon_file = self.ui.icon_file.text()
        pixmap = QPixmap(icon_file).scaled(16,16,Qt.KeepAspectRatio,Qt.FastTransformation)
        self.ui.icon.setPixmap(pixmap)

    def get_value(self):
        bookmark = {}

        bookmark['directory'] = self.ui.location.text().replace('\\','/')
        bookmark['name'] = self.ui.name.text()
        bookmark['icon'] = self.ui.icon_file.text()
        bookmark['description'] = self.ui.description.text()
        bookmark['option'] = {}
        bookmark['option']['recursive'] = self.ui.recursive.isChecked()
        bookmark['option']['apply_filter'] = self.ui.apply_filter.isChecked()
        bookmark['option']['filter_word'] = self.ui.search.text()
        bookmark['option']['filter_exclusion'] = self.ui.exclusion.text()
        bookmark['option']['filter_ext'] = self.ui.filter.currentText()
        bookmark['option']['latest'] = self.ui.latest.isChecked()
        bookmark['option']['apply_doubleclick'] = self.ui.apply_doubleclick.isChecked()
        bookmark['option']['dc_action'] = self.dc_actions.checkedId()

        return bookmark

    @staticmethod
    def open_dialog(parent=None,bookmark=None):
        dialog = btkFileManagerOptionDialog(parent)
        dialog.set_default(bookmark)
        result = dialog.exec_()
        ret = dialog.get_value()
        return (result == QDialog.Accepted,ret)

class Ui_btkMayaFileManager(object):
    def setupUi(self, btkMayaFileManager):
        if not btkMayaFileManager.objectName():
            btkMayaFileManager.setObjectName(u"btkMayaFileManager")
        btkMayaFileManager.resize(765, 633)
        self.centralwidget = QWidget(btkMayaFileManager)
        self.centralwidget.setObjectName(u"centralwidget")
        self.verticalLayout_3 = QVBoxLayout(self.centralwidget)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.splitter = QSplitter(self.centralwidget)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setOrientation(Qt.Horizontal)
        self.splitter.setOpaqueResize(True)
        self.splitter.setHandleWidth(10)
        self.splitter.setChildrenCollapsible(False)
        self.layoutWidget = QWidget(self.splitter)
        self.layoutWidget.setObjectName(u"layoutWidget")
        self.verticalLayout = QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_6 = QHBoxLayout()
        self.horizontalLayout_6.setSpacing(2)
        self.horizontalLayout_6.setObjectName(u"horizontalLayout_6")
        self.label_2 = QLabel(self.layoutWidget)
        self.label_2.setObjectName(u"label_2")

        self.horizontalLayout_6.addWidget(self.label_2)

        self.group = QComboBox(self.layoutWidget)
        self.group.addItem("")
        self.group.setObjectName(u"group")

        self.horizontalLayout_6.addWidget(self.group)

        self.add_group = QPushButton(self.layoutWidget)
        self.add_group.setObjectName(u"add_group")
        self.add_group.setMaximumSize(QSize(50, 16777215))
        icon = QIcon()
        icon.addFile(u"icon/plus.png", QSize(), QIcon.Normal, QIcon.Off)
        self.add_group.setIcon(icon)

        self.horizontalLayout_6.addWidget(self.add_group)

        self.remove_group = QPushButton(self.layoutWidget)
        self.remove_group.setObjectName(u"remove_group")
        self.remove_group.setMaximumSize(QSize(50, 16777215))
        icon1 = QIcon()
        icon1.addFile(u"icon/minus.png", QSize(), QIcon.Normal, QIcon.Off)
        self.remove_group.setIcon(icon1)

        self.horizontalLayout_6.addWidget(self.remove_group)

        self.horizontalLayout_6.setStretch(1, 1)

        self.verticalLayout.addLayout(self.horizontalLayout_6)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setSpacing(2)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.add_dir = QPushButton(self.layoutWidget)
        self.add_dir.setObjectName(u"add_dir")
        icon2 = QIcon()
        icon2.addFile(u"icon/add.png", QSize(), QIcon.Normal, QIcon.Off)
        self.add_dir.setIcon(icon2)

        self.horizontalLayout.addWidget(self.add_dir)

        self.remove_dir = QPushButton(self.layoutWidget)
        self.remove_dir.setObjectName(u"remove_dir")
        icon3 = QIcon()
        icon3.addFile(u"icon/remove.png", QSize(), QIcon.Normal, QIcon.Off)
        self.remove_dir.setIcon(icon3)

        self.horizontalLayout.addWidget(self.remove_dir)

        self.separator = QPushButton(self.layoutWidget)
        self.separator.setObjectName(u"separator")
        icon4 = QIcon()
        icon4.addFile(u"icon/separator.png", QSize(), QIcon.Normal, QIcon.Off)
        self.separator.setIcon(icon4)
        self.separator.setIconSize(QSize(16, 16))

        self.horizontalLayout.addWidget(self.separator)

        self.up = QPushButton(self.layoutWidget)
        self.up.setObjectName(u"up")
        icon5 = QIcon()
        icon5.addFile(u"icon/to_up.png", QSize(), QIcon.Normal, QIcon.Off)
        self.up.setIcon(icon5)

        self.horizontalLayout.addWidget(self.up)

        self.down = QPushButton(self.layoutWidget)
        self.down.setObjectName(u"down")
        icon6 = QIcon()
        icon6.addFile(u"icon/to_down.png", QSize(), QIcon.Normal, QIcon.Off)
        self.down.setIcon(icon6)

        self.horizontalLayout.addWidget(self.down)

        self.option = QPushButton(self.layoutWidget)
        self.option.setObjectName(u"option")
        icon7 = QIcon()
        icon7.addFile(u"icon/option.png", QSize(), QIcon.Normal, QIcon.Off)
        self.option.setIcon(icon7)

        self.horizontalLayout.addWidget(self.option)


        self.verticalLayout.addLayout(self.horizontalLayout)

        self.bookmarks = QTableWidget(self.layoutWidget)
        self.bookmarks.setObjectName(u"bookmarks")
        self.bookmarks.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.bookmarks.setProperty("showDropIndicator", False)
        self.bookmarks.setDragDropMode(QAbstractItemView.DropOnly)
        self.bookmarks.setSelectionMode(QAbstractItemView.SingleSelection)
        self.bookmarks.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.bookmarks.setShowGrid(False)
        self.bookmarks.horizontalHeader().setVisible(False)
        self.bookmarks.horizontalHeader().setStretchLastSection(True)
        self.bookmarks.verticalHeader().setVisible(False)
        self.bookmarks.verticalHeader().setDefaultSectionSize(20)

        self.verticalLayout.addWidget(self.bookmarks)

        self.splitter.addWidget(self.layoutWidget)
        self.layoutWidget1 = QWidget(self.splitter)
        self.layoutWidget1.setObjectName(u"layoutWidget1")
        self.verticalLayout_2 = QVBoxLayout(self.layoutWidget1)
        self.verticalLayout_2.setSpacing(3)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setSpacing(2)
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.upper_directory = QPushButton(self.layoutWidget1)
        self.upper_directory.setObjectName(u"upper_directory")
        icon8 = QIcon()
        icon8.addFile(u"icon/up.png", QSize(), QIcon.Normal, QIcon.Off)
        self.upper_directory.setIcon(icon8)

        self.horizontalLayout_7.addWidget(self.upper_directory)

        self.location = QLineEdit(self.layoutWidget1)
        self.location.setObjectName(u"location")

        self.horizontalLayout_7.addWidget(self.location)

        self.open_filedialog = QPushButton(self.layoutWidget1)
        self.open_filedialog.setObjectName(u"open_filedialog")
        icon9 = QIcon()
        icon9.addFile(u"icon/open.png", QSize(), QIcon.Normal, QIcon.Off)
        self.open_filedialog.setIcon(icon9)

        self.horizontalLayout_7.addWidget(self.open_filedialog)


        self.verticalLayout_2.addLayout(self.horizontalLayout_7)

        self.frame = QFrame(self.layoutWidget1)
        self.frame.setObjectName(u"frame")
        self.frame.setFrameShape(QFrame.StyledPanel)
        self.frame.setFrameShadow(QFrame.Plain)
        self.horizontalLayout_3 = QHBoxLayout(self.frame)
        self.horizontalLayout_3.setContentsMargins(2, 2, 2, 2)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.apply_filter = QCheckBox(self.frame)
        self.apply_filter.setObjectName(u"apply_filter")

        self.horizontalLayout_3.addWidget(self.apply_filter)

        self.filters = QFrame(self.frame)
        self.filters.setObjectName(u"filters")
        self.filters.setEnabled(True)
        self.filters.setFrameShape(QFrame.StyledPanel)
        self.filters.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_10 = QHBoxLayout(self.filters)
        self.horizontalLayout_10.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_10.setObjectName(u"horizontalLayout_10")
        self.verticalLayout_5 = QVBoxLayout()
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.horizontalLayout_12 = QHBoxLayout()
        self.horizontalLayout_12.setObjectName(u"horizontalLayout_12")
        self.filter = QComboBox(self.filters)
        self.filter.addItem("")
        self.filter.addItem("")
        self.filter.addItem("")
        self.filter.addItem("")
        self.filter.setObjectName(u"filter")
        self.filter.setMinimumSize(QSize(0, 0))

        self.horizontalLayout_12.addWidget(self.filter)

        self.horizontalSpacer_3 = QSpacerItem(18, 20, QSizePolicy.Fixed, QSizePolicy.Minimum)

        self.horizontalLayout_12.addItem(self.horizontalSpacer_3)

        self.latest = QCheckBox(self.filters)
        self.latest.setObjectName(u"latest")

        self.horizontalLayout_12.addWidget(self.latest)

        self.horizontalSpacer_6 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_12.addItem(self.horizontalSpacer_6)


        self.verticalLayout_5.addLayout(self.horizontalLayout_12)

        self.horizontalLayout_11 = QHBoxLayout()
        self.horizontalLayout_11.setSpacing(2)
        self.horizontalLayout_11.setObjectName(u"horizontalLayout_11")
        self.search = QLineEdit(self.filters)
        self.search.setObjectName(u"search")

        self.horizontalLayout_11.addWidget(self.search)

        self.exclusion = QLineEdit(self.filters)
        self.exclusion.setObjectName(u"exclusion")

        self.horizontalLayout_11.addWidget(self.exclusion)


        self.verticalLayout_5.addLayout(self.horizontalLayout_11)


        self.horizontalLayout_10.addLayout(self.verticalLayout_5)


        self.horizontalLayout_3.addWidget(self.filters)


        self.verticalLayout_2.addWidget(self.frame)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setSpacing(2)
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.recursive = QPushButton(self.layoutWidget1)
        self.recursive.setObjectName(u"recursive")
        icon10 = QIcon()
        icon10.addFile(u"icon/recursive.png", QSize(), QIcon.Normal, QIcon.Off)
        self.recursive.setIcon(icon10)
        self.recursive.setCheckable(True)

        self.horizontalLayout_4.addWidget(self.recursive)

        self.sort_each_folder = QCheckBox(self.layoutWidget1)
        self.sort_each_folder.setObjectName(u"sort_each_folder")
        self.sort_each_folder.setChecked(True)

        self.horizontalLayout_4.addWidget(self.sort_each_folder)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_4.addItem(self.horizontalSpacer)

        self.reload = QPushButton(self.layoutWidget1)
        self.reload.setObjectName(u"reload")
        icon11 = QIcon()
        icon11.addFile(u"icon/reload.png", QSize(), QIcon.Normal, QIcon.Off)
        self.reload.setIcon(icon11)

        self.horizontalLayout_4.addWidget(self.reload)


        self.verticalLayout_2.addLayout(self.horizontalLayout_4)

        self.files = QTableWidget(self.layoutWidget1)
        self.files.setObjectName(u"files")
        self.files.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.files.setProperty("showDropIndicator", False)
        self.files.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.files.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.files.setColumnCount(0)
        self.files.horizontalHeader().setVisible(True)
        self.files.horizontalHeader().setProperty("showSortIndicator", True)
        self.files.horizontalHeader().setStretchLastSection(True)
        self.files.verticalHeader().setVisible(False)

        self.verticalLayout_2.addWidget(self.files)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setSpacing(2)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.open_file = QPushButton(self.layoutWidget1)
        self.open_file.setObjectName(u"open_file")
        self.open_file.setMinimumSize(QSize(0, 25))

        self.horizontalLayout_2.addWidget(self.open_file)

        self.import_file = QPushButton(self.layoutWidget1)
        self.import_file.setObjectName(u"import_file")
        self.import_file.setMinimumSize(QSize(0, 25))

        self.horizontalLayout_2.addWidget(self.import_file)

        self.reference_file = QPushButton(self.layoutWidget1)
        self.reference_file.setObjectName(u"reference_file")
        self.reference_file.setMinimumSize(QSize(0, 25))

        self.horizontalLayout_2.addWidget(self.reference_file)

        self.gpucache_file = QPushButton(self.layoutWidget1)
        self.gpucache_file.setObjectName(u"gpucache_file")
        self.gpucache_file.setMinimumSize(QSize(0, 25))

        self.horizontalLayout_2.addWidget(self.gpucache_file)

        self.open_explorer = QPushButton(self.layoutWidget1)
        self.open_explorer.setObjectName(u"open_explorer")
        sizePolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.open_explorer.sizePolicy().hasHeightForWidth())
        self.open_explorer.setSizePolicy(sizePolicy)
        self.open_explorer.setMinimumSize(QSize(25, 25))
        self.open_explorer.setMaximumSize(QSize(25, 25))
        icon12 = QIcon()
        icon12.addFile(u"icon/explorer.png", QSize(), QIcon.Normal, QIcon.Off)
        self.open_explorer.setIcon(icon12)

        self.horizontalLayout_2.addWidget(self.open_explorer)


        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.horizontalLayout_8 = QHBoxLayout()
        self.horizontalLayout_8.setObjectName(u"horizontalLayout_8")
        self.set_project = QCheckBox(self.layoutWidget1)
        self.set_project.setObjectName(u"set_project")

        self.horizontalLayout_8.addWidget(self.set_project)

        self.horizontalSpacer_4 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_8.addItem(self.horizontalSpacer_4)

        self.horizontalLayout_9 = QHBoxLayout()
        self.horizontalLayout_9.setObjectName(u"horizontalLayout_9")
        self.repeat_enable = QCheckBox(self.layoutWidget1)
        self.repeat_enable.setObjectName(u"repeat_enable")

        self.horizontalLayout_9.addWidget(self.repeat_enable)

        self.repeat_num = BTKSpinBoxEditSlideVari(self.layoutWidget1)
        self.repeat_num.setObjectName(u"repeat_num")
        self.repeat_num.setMinimum(1)

        self.horizontalLayout_9.addWidget(self.repeat_num)


        self.horizontalLayout_8.addLayout(self.horizontalLayout_9)


        self.verticalLayout_2.addLayout(self.horizontalLayout_8)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setSpacing(2)
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.label_doubleclick = QLabel(self.layoutWidget1)
        self.label_doubleclick.setObjectName(u"label_doubleclick")

        self.horizontalLayout_5.addWidget(self.label_doubleclick)

        self.dc_none = QRadioButton(self.layoutWidget1)
        self.dc_none.setObjectName(u"dc_none")
        self.dc_none.setChecked(True)

        self.horizontalLayout_5.addWidget(self.dc_none)

        self.dc_open = QRadioButton(self.layoutWidget1)
        self.dc_open.setObjectName(u"dc_open")

        self.horizontalLayout_5.addWidget(self.dc_open)

        self.dc_import = QRadioButton(self.layoutWidget1)
        self.dc_import.setObjectName(u"dc_import")

        self.horizontalLayout_5.addWidget(self.dc_import)

        self.dc_reference = QRadioButton(self.layoutWidget1)
        self.dc_reference.setObjectName(u"dc_reference")

        self.horizontalLayout_5.addWidget(self.dc_reference)

        self.dc_gpucache = QRadioButton(self.layoutWidget1)
        self.dc_gpucache.setObjectName(u"dc_gpucache")

        self.horizontalLayout_5.addWidget(self.dc_gpucache)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_5.addItem(self.horizontalSpacer_2)


        self.verticalLayout_2.addLayout(self.horizontalLayout_5)

        self.horizontalLayout_13 = QHBoxLayout()
        self.horizontalLayout_13.setObjectName(u"horizontalLayout_13")
        self.reference_editor = QPushButton(self.layoutWidget1)
        self.reference_editor.setObjectName(u"reference_editor")

        self.horizontalLayout_13.addWidget(self.reference_editor)

        self.horizontalSpacer_5 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_13.addItem(self.horizontalSpacer_5)


        self.verticalLayout_2.addLayout(self.horizontalLayout_13)

        self.splitter.addWidget(self.layoutWidget1)

        self.verticalLayout_3.addWidget(self.splitter)

        btkMayaFileManager.setCentralWidget(self.centralwidget)
        self.statusBar = QStatusBar(btkMayaFileManager)
        self.statusBar.setObjectName(u"statusBar")
        btkMayaFileManager.setStatusBar(self.statusBar)

        self.retranslateUi(btkMayaFileManager)

        QMetaObject.connectSlotsByName(btkMayaFileManager)
    # setupUi

    def retranslateUi(self, btkMayaFileManager):
        btkMayaFileManager.setWindowTitle(QCoreApplication.translate("btkMayaFileManager", u"MainWindow", None))
        self.label_2.setText(QCoreApplication.translate("btkMayaFileManager", u"Group", None))
        self.group.setItemText(0, QCoreApplication.translate("btkMayaFileManager", u"default", None))

        self.add_group.setText("")
        self.remove_group.setText("")
        self.add_dir.setText("")
        self.remove_dir.setText("")
        self.separator.setText("")
        self.up.setText("")
        self.down.setText("")
        self.option.setText("")
        self.upper_directory.setText("")
        self.open_filedialog.setText("")
        self.apply_filter.setText(QCoreApplication.translate("btkMayaFileManager", u"Filter", None))
        self.filter.setItemText(0, QCoreApplication.translate("btkMayaFileManager", u"ma, mb, abc", None))
        self.filter.setItemText(1, QCoreApplication.translate("btkMayaFileManager", u"ma, mb", None))
        self.filter.setItemText(2, QCoreApplication.translate("btkMayaFileManager", u"abc", None))
        self.filter.setItemText(3, QCoreApplication.translate("btkMayaFileManager", u"*", None))

        self.latest.setText(QCoreApplication.translate("btkMayaFileManager", u"Latest version only", None))
        self.search.setPlaceholderText(QCoreApplication.translate("btkMayaFileManager", u"Keyword", None))
        self.exclusion.setPlaceholderText(QCoreApplication.translate("btkMayaFileManager", u"Keywords to exclude", None))
        self.recursive.setText("")
        self.sort_each_folder.setText(QCoreApplication.translate("btkMayaFileManager", u"Sort files within each folder", None))
        self.reload.setText("")
        self.open_file.setText(QCoreApplication.translate("btkMayaFileManager", u"Open", None))
        self.import_file.setText(QCoreApplication.translate("btkMayaFileManager", u"Import", None))
        self.reference_file.setText(QCoreApplication.translate("btkMayaFileManager", u"Reference", None))
        self.gpucache_file.setText(QCoreApplication.translate("btkMayaFileManager", u"GPUCache", None))
        self.open_explorer.setText("")
        self.set_project.setText(QCoreApplication.translate("btkMayaFileManager", u"Set the project when open the scene.", None))
        self.repeat_enable.setText(QCoreApplication.translate("btkMayaFileManager", u"Repeat", None))
        self.label_doubleclick.setText(QCoreApplication.translate("btkMayaFileManager", u"Dbl Clk", None))
        self.dc_none.setText(QCoreApplication.translate("btkMayaFileManager", u"Non", None))
        self.dc_open.setText(QCoreApplication.translate("btkMayaFileManager", u"Opn", None))
        self.dc_import.setText(QCoreApplication.translate("btkMayaFileManager", u"Imp", None))
        self.dc_reference.setText(QCoreApplication.translate("btkMayaFileManager", u"Ref", None))
        self.dc_gpucache.setText(QCoreApplication.translate("btkMayaFileManager", u"GPU", None))
        self.reference_editor.setText(QCoreApplication.translate("btkMayaFileManager", u"Reference Editor", None))
    # retranslateUi

class btkFileManager(QtWidgets.QMainWindow):
    _instance = None

    titleName = 'btk File Manager'

    ptr = OpenMayaUI.MQtUtil.mainWindow()
    parent = shiboken.wrapInstance(int(ptr), QWidget)

    def __init__(self, parent=parent):
        super(btkFileManager, self).__init__(parent)

        self.setObjectName(self.__class__.titleName)
        self.ui=Ui_btkMayaFileManager()

        wd = os.getcwd()                    #作業ディレクトリをいったん保存
        os.chdir(os.path.dirname(__file__)+'/ui') #作業ディレクトリをPythonファイルの場所に指定　※アイコンなどは作業ディレクトリからの相対パスで読み込まれるため
        self.ui.setupUi(self)               #相対パスを使ったアイコンなどの設定がされる
        os.chdir(wd)                        #作業ディレクトリを元に戻しておく
        self.setWindowTitle(self.titleName)

        thread = threading.Thread(target=self.setup_file_icon,daemon=True)
        thread.start()

        self.all_files = []
        self.memo = {}
        self.files = [] #list of dict ( {'file_path': ,''(color): ,'Filename': ,'Parent': ,'Date': ,'Size': ,'Memo': } )

        self.block_filter = False
        self.cur_group = 'default'
        self.clear_action = {'search':None,'exclusion':None}

        self.sort = ['Filename',Qt.AscendingOrder]

        self.dc_actions = QButtonGroup(self)
        self.dc_actions.addButton(self.ui.dc_none,0)
        self.dc_actions.addButton(self.ui.dc_open,1)
        self.dc_actions.addButton(self.ui.dc_import,2)
        self.dc_actions.addButton(self.ui.dc_reference,3)
        self.dc_actions.addButton(self.ui.dc_gpucache,4)
        self.ui.open_file.setEnabled(False)
        self.ui.import_file.setEnabled(False)
        self.ui.reference_file.setEnabled(False)
        self.ui.gpucache_file.setEnabled(False)
        self.ui.open_explorer.setEnabled(False)
        self.dc_actions.buttonClicked.connect(self.set_mouse_cursor)

        self.ui.bookmarks.setAcceptDrops(True)

        self.ui.files.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.ui.repeat_num.menu_setup(max_digit=2)
        
        self.setup_context()
        self.setup_mouse_cursor()

        self.ui.splitter.splitterMoved.connect(self.sizeChanged)

        self.ui.group.currentIndexChanged.connect(self.refresh)
        self.ui.add_group.clicked.connect(self.add_group)
        self.ui.remove_group.clicked.connect(self.remove_group)

        self.ui.add_dir.clicked.connect(self.add_bookmark)
        self.ui.remove_dir.clicked.connect(self.remove_bookmark)
        self.ui.separator.clicked.connect(self.add_separator)
        self.ui.up.clicked.connect(partial(self.up_down,mode='up'))
        self.ui.down.clicked.connect(partial(self.up_down,mode='down'))
        self.ui.option.clicked.connect(self.set_option)

        self.ui.bookmarks.itemSelectionChanged.connect(self.directory_selected)
        self.ui.bookmarks.doubleClicked.connect(self.open_explorer_from_bookmark)

        self.ui.files.itemSelectionChanged.connect(self.file_selected)
        self.ui.files.doubleClicked.connect(self.file_doubleclicked)
        self.ui.files.horizontalHeader().sectionClicked.connect(self.sorted)

        self.ui.upper_directory.clicked.connect(self.upper_directory)
        self.ui.open_filedialog.clicked.connect(self.open_filedialog)

        self.ui.location.textChanged.connect(self.directory_changed)
        self.ui.apply_filter.toggled.connect(self.set_file_list)
        self.ui.filter.currentIndexChanged.connect(self.set_file_list)
        self.ui.search.textChanged.connect(self.filter_linedit_changed)
        self.ui.exclusion.textChanged.connect(self.filter_linedit_changed)
        self.ui.latest.toggled.connect(self.set_file_list)
        self.ui.recursive.clicked.connect(self.directory_changed)
        self.ui.sort_each_folder.clicked.connect(self.set_file_list)
        self.ui.open_explorer.clicked.connect(self.open_explorer_from_file)
        self.ui.reload.clicked.connect(self.directory_changed)

        self.ui.open_file.clicked.connect(partial(self.open_file,mode='open'))
        self.ui.import_file.clicked.connect(partial(self.open_file,mode='import'))
        self.ui.reference_file.clicked.connect(partial(self.open_file,mode='reference'))
        self.ui.gpucache_file.clicked.connect(partial(self.open_file,mode='gpucache'))

        self.ui.reference_editor.clicked.connect(self.open_reference_editor)

### CONFIG  FILE
    def save_config(self):
        with open(user_config_file,'w') as f:
            config = {}

            config['bookmarks'] = self.bookmarks
            config_ui = {}

            window_geo = self.geometry()
            config_ui['windowX'] = window_geo.x()
            config_ui['windowY'] = window_geo.y()
            config_ui['windowW'] = window_geo.width()
            config_ui['windowH'] = window_geo.height()
            config_ui['splitter_size'] = self.ui.splitter.sizes()

            config_ui['current_group'] = self.ui.group.currentText()
            row = self.ui.bookmarks.currentRow()
            if row == -1:
                row = 0
            config_ui['last_selection'] = row
            config_ui['filter'] = self.ui.filter.currentIndex()

            config_ui['sort_section'] = self.sort[0]
            if self.sort[1] == Qt.AscendingOrder:
                config_ui['sort_order'] = 0
            else:
                config_ui['sort_order'] = 1

            config_ui['set_project'] = self.ui.set_project.isChecked()
            config_ui['dc_action'] = self.dc_actions.checkedId()
            config['ui'] = config_ui
            json.dump(config,f,indent=4)

    def load_ui_default(self):
        if os.path.exists(user_config_file):
            with open(user_config_file,'r') as f:
                config_ui=json.load(f)['ui']
                window_geo = self.geometry()
                if config_ui.get('windowX'):
                    window_geo.setX(config_ui.get('windowX'))
                if config_ui.get('windowY'):
                    window_geo.setY( config_ui.get('windowY') )
                if config_ui.get('windowW'):
                    window_geo.setWidth( config_ui.get('windowW') )
                if config_ui.get('windowH'):
                    window_geo.setHeight( config_ui.get('windowH') )
                self.setGeometry(window_geo)

                if config_ui.get('splitter_size'):
                    self.ui.splitter.setSizes(config_ui.get('splitter_size'))
                self.ui.group.setCurrentText(config_ui.get('current_group','default'))
                self.last_selection = config_ui.get('last_selection',0)
                self.ui.filter.setCurrentIndex(config_ui.get('filter',0))
                self.sort[0] = config_ui.get('sort_section','Filename')
                sort_order = config_ui.get('sort_order',0)
                if sort_order == 0:
                    self.sort[1] = Qt.AscendingOrder
                else:
                    self.sort[1] = Qt.DescendingOrder 
                self.ui.set_project.setChecked(config_ui.get('set_project',0))
                self.dc_actions.button(config_ui.get('dc_action',0)).setChecked(True)

    def load_bookmarks_from_file(self):
        if os.path.exists(user_config_file):
            with open(user_config_file,'r') as f:
                self.bookmarks=json.load(f)['bookmarks']
        else:
            self.bookmarks = {}
            self.bookmarks['default'] = []

### SETUPS
    def setup_file_icon(self):
        pixmap = QPixmap(f'{icon_dir}/file/none.png')
        self.file_icon_none = QIcon(pixmap)

        self.file_icon = {}
        files = glob.glob(f'{icon_dir}/file/*.png')
        for file in files:
            key = os.path.splitext(os.path.basename(file))[0]
            pixmap = QPixmap(file)
            self.file_icon[key] = QIcon(pixmap)
            # self.file_icon[key] = QIcon(file)

    def setup_mouse_cursor(self):
        self.cursors = {}
        for _type in ['none','open','import','reference','gpucache']:
            img_file = f'{icon_dir}/cursor/{_type}.png'
            self.cursors[_type] = QCursor(QPixmap(img_file))

    def setup_context(self):
        self.file_context = QMenu(self)

        label = QLabel('Coloring')
        action = QWidgetAction(self)
        action.setDefaultWidget(label)
        self.file_context.addAction(action)

        files = glob.glob(f'{icon_dir}/context/*.png')
        for file in files:
            icon = QIcon(file)
            color_name = os.path.splitext(os.path.basename(file))[0].split('_')[-1]
            action = QAction(icon,color_name,self)
            action.triggered.connect(self.context_clicked)
            self.file_context.addAction(action)

        label = QLabel('Memo')
        action = QWidgetAction(self)
        action.setDefaultWidget(label)
        self.file_context.addAction(action)

        add_memo = QAction('Add memo',self)
        add_memo.triggered.connect(self.context_memo_clicked)
        self.file_context.addAction(add_memo)
        edit_memo = QAction('Edit memo',self)
        edit_memo.triggered.connect(self.context_memo_clicked)
        self.file_context.addAction(edit_memo)
        delete_memo = QAction('Delete memo',self)
        delete_memo.triggered.connect(self.context_memo_clicked)
        self.file_context.addAction(delete_memo)

        self.ui.files.setContextMenuPolicy(Qt.CustomContextMenu)
        self.ui.files.customContextMenuRequested.connect(self.open_file_context)

    def set_mouse_cursor(self):
        _type = self.dc_actions.checkedButton().objectName().split('_')[-1]
        self.ui.files.setCursor(self.cursors[_type])

### CONTEXT MENU

    def open_file_context(self,position):
        row = self.ui.bookmarks.currentRow()-1
        if row >= 0:
            bookmark = self.bookmarks[self.cur_group][row]
            if bookmark.get('directory') is None:
                return

            self.file_context.exec_(self.ui.files.mapToGlobal(position))

    def context_memo_clicked(self):
        row = self.ui.bookmarks.currentRow()-1
        if row >= 0:
            sender = self.sender()
            text = sender.text()

            file_rows = []
            for idx in self.ui.files.selectedIndexes():
                if idx.row() not in file_rows:
                    file_rows.append(idx.row())
            if len(file_rows) == 0:
                return

            memo_text = ''

            bookmark = self.bookmarks[self.cur_group][row]
            file_memo = bookmark.get('file_memo',{})
            if text == 'Add memo':
                memo_text , result = QInputDialog.getText(self,'Memo','Add a note.',QLineEdit.Normal,'')
                if result:
                    for file_row in file_rows:
                        file = self.files[file_row]['file_path']
                        memo = file_memo.get(file,{})
                        memo['text'] = memo_text
                        file_memo[file] = memo
            elif text == 'Delete memo':
                for file_row in file_rows:
                    file = self.files[file_row]['file_path']
                    memo = file_memo.get(file,{})
                    memo['text'] = ''
                    file_memo[file] = memo
            elif text == 'Edit memo':
                file = self.files[0]['file_path']
                memo = file_memo.get(file,{})
                cur = memo.get('text','')
                memo_text , result = QInputDialog.getText(self,'Memo','Edit a note.',QLineEdit.Normal,cur)
                if result:
                    for file_row in file_rows:
                        file = self.files[file_row]['file_path']
                        memo = file_memo.get(file,{})
                        memo['text'] = memo_text
                        file_memo[file] = memo
            self.memo = file_memo
            self.directory_changed()

    def context_clicked(self):
        row = self.ui.bookmarks.currentRow()-1
        if row >= 0:
            sender = self.sender()
            color = sender.text().lower()

            file_rows = []
            for idx in self.ui.files.selectedIndexes():
                if idx.row() not in file_rows:
                    file_rows.append(idx.row())
            if len(file_rows) == 0:
                return

            bookmark = self.bookmarks[self.cur_group][row]
            file_memo = bookmark.get('file_memo',{})

            for file_row in file_rows:
                file = self.files[file_row]['file_path']
                memo = file_memo.get(file,{})
                memo['color'] = color
                file_memo[file] = memo
            
            self.memo = file_memo
        
            self.directory_changed()

### EVENTS
    def showEvent(self,event):
        self.load_bookmarks_from_file()
        self.refresh()
        self.load_ui_default()
        self.sizeChanged()
        self.ui.bookmarks.selectRow(self.last_selection)
        self.set_mouse_cursor()

    def closeEvent(self,event):
        self.save_config()

    def resizeEvent(self,event):
        self.sizeChanged()

    def sizeChanged(self):
        size = self.ui.splitter.sizes()[1]
        if size < 280:
            self.ui.label_doubleclick.setText('DClk:')
            self.ui.dc_none.setText('N')
            self.ui.dc_open.setText('O')
            self.ui.dc_import.setText('I')
            self.ui.dc_reference.setText('R')
            self.ui.dc_gpucache.setText('G')
        elif size < 400:
            self.ui.open_file.setText('Opn')
            self.ui.import_file.setText('Imp')
            self.ui.reference_file.setText('Ref')
            self.ui.gpucache_file.setText('GPU')

            self.ui.label_doubleclick.setText('Dbl Clk:')
            self.ui.dc_none.setText('Non')
            self.ui.dc_open.setText('Opn')
            self.ui.dc_import.setText('Imp')
            self.ui.dc_reference.setText('Ref')
            self.ui.dc_gpucache.setText('GPU')
        else:
            self.ui.open_file.setText('Open')
            self.ui.import_file.setText('Import')
            self.ui.reference_file.setText('Reference')
            self.ui.gpucache_file.setText('GPUCache')

            self.ui.label_doubleclick.setText('Double Click:')
            self.ui.dc_none.setText('None')
            self.ui.dc_open.setText('Open')
            self.ui.dc_import.setText('Import')
            self.ui.dc_reference.setText('Reference')
            self.ui.dc_gpucache.setText('GPUCache')

        if size < 310:
            self.ui.set_project.setText('Set project')
        else:
            self.ui.set_project.setText('Set the project when open the scene.')

### GUI REFRESH
    def refresh(self):
        self.cur_group = self.ui.group.currentText()

        self.ui.group.blockSignals(True)

        self.ui.group.clear()
        self.ui.group.addItems(self.bookmarks.keys())
        if self.ui.group.findText(self.cur_group) == -1:
            self.cur_group = 'default'        
        self.ui.group.setCurrentText(self.cur_group)
        self.ui.group.blockSignals(False)

        self.ui.bookmarks.clear()
        self.ui.bookmarks.setColumnCount(3)
        self.ui.bookmarks.setRowCount(len(self.bookmarks[self.cur_group])+1)

        self.ui.bookmarks.setItem( 0, 0, QTableWidgetItem(QIcon(f'{icon_dir}/none.png'), '  Current  '))

        for _row,bookmark in enumerate(self.bookmarks[self.cur_group]):
            row = _row + 1
            name = bookmark.get('name')
            description = bookmark.get('description','')
            if name is None:
                # self.ui.directories.addItem('----------')
                self.ui.bookmarks.setItem( row, 0, QTableWidgetItem('   ----------   ') )
            else:
                # self.ui.directories.addItem(name)
                icon_file = bookmark.get('icon')
                if icon_file:
                    self.ui.bookmarks.setItem( row, 0, QTableWidgetItem(QIcon(icon_file), f'  {name}  '))
                else:
                    self.ui.bookmarks.setItem( row, 0, QTableWidgetItem(self.file_icon_none, f'  {name}  ') )
                self.ui.bookmarks.setItem( row, 1, QTableWidgetItem(f'  {description}  ') )

        self.ui.bookmarks.horizontalHeader().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)

    def block_filtering(self,block):
        self.ui.apply_filter.blockSignals(block)
        self.ui.filter.blockSignals(block)
        self.ui.latest.blockSignals(block)

        self.block_filter = block

    def save_and_refresh(self):
        self.save_config()
        self.refresh()

### GROUPS
    def add_group(self):
        new_group_name, ok = QInputDialog.getText(self, 'New Group', 'Group Name', QLineEdit.Normal, 'GROUP')
        if ok:
            if new_group_name in self.bookmarks.keys():
                QMessageBox.warning(self,'Error',f'{new_group_name} is already exists.',QMessageBox.Ok)
                return
            self.bookmarks[new_group_name] = []
            self.save_and_refresh()
            self.ui.group.setCurrentText(new_group_name)

    def remove_group(self):
        if self.cur_group == 'default':
            QMessageBox.warning(self,'Error','Cannot remove the default.',QMessageBox.Ok)
            return

        ret = QMessageBox.question(self,'Confirm',f'Remove {self.cur_group} OK?',QMessageBox.Ok, QMessageBox.Cancel)
        if ret == QMessageBox.Ok:
            del self.bookmarks[self.cur_group]
            self.save_and_refresh()

### BOOKMARK_LIST_OPERATIONS
    def add_bookmark(self):
        root_dir = cmds.workspace( q=True, fullName=True )
        cur_dir = f'{root_dir}/scenes'
        if not os.path.isdir(cur_dir):
            cur_dir = root_dir
        path = QFileDialog.getExistingDirectory(self,'Select Directory',cur_dir)
        if path:
            bookmark = {}
            bookmark['directory'] = path.replace('\\','/')
            bookmark['name'] = os.path.basename(path)
            bookmark['option'] = {}
            self.bookmarks[self.cur_group].append(bookmark)
        self.save_and_refresh()

    def remove_bookmark(self):
        row = self.ui.bookmarks.currentRow()
        if row <= 0:
            return

        del self.bookmarks[self.cur_group][row-1]
        self.save_and_refresh()

    def add_separator(self):
        bookmark = {}
        self.bookmarks[self.cur_group].append(bookmark)

        self.save_and_refresh()

    def up_down(self,mode='up'):
        row = self.ui.bookmarks.currentRow()-1
        if row < 0:
            return

        if mode == 'up':
            if row > 0:
                self.bookmarks[self.cur_group][row-1], self.bookmarks[self.cur_group][row] = self.bookmarks[self.cur_group][row], self.bookmarks[self.cur_group][row-1]
        elif mode == 'down':
            if self.ui.bookmarks.rowCount()-1 > row+1:
                self.bookmarks[self.cur_group][row+1], self.bookmarks[self.cur_group][row] = self.bookmarks[self.cur_group][row], self.bookmarks[self.cur_group][row+1]
                row += 2
            else:
                row += 1
        self.save_and_refresh()
        self.ui.bookmarks.setCurrentCell(row,0)

    def set_option(self):
        row = self.ui.bookmarks.currentRow() - 1
        if row >= 0:
            bookmark = self.bookmarks[self.cur_group][row]
            if bookmark.get('directory') is None:
                return
            
            ok,new_bookmark = btkFileManagerOptionDialog.open_dialog(self,bookmark)
            if ok:
                self.bookmarks[self.cur_group][row] = new_bookmark
                self.save_and_refresh()
        self.ui.bookmarks.setCurrentCell(row,0)

    def directory_selected(self):
        row = self.ui.bookmarks.currentRow()
        self.ui.location.setText('')

        if row == -1:
            self.clear_files()
            return
        
        if row == 0:
            self.clear_files()
            self.ui.recursive.setChecked(False)
            root_dir = cmds.workspace( q=True, fullName=True )
            cur_dir = f'{root_dir}/scenes'
            if not os.path.isdir(cur_dir):
                cur_dir = root_dir

            self.block_filtering(True)
            self.ui.apply_filter.setChecked(False)
            self.ui.search.setText('')
            self.ui.exclusion.setText('')
            self.ui.filter.setCurrentText('')
            self.ui.latest.setChecked(False)
            self.block_filtering(False)

            self.ui.location.setText(cur_dir)
            return

        if row > len(self.bookmarks[self.cur_group]):
            self.clear_files()
            return

        row -= 1

        bookmark = self.bookmarks[self.cur_group][row]
        dir = bookmark.get('directory','')
        self.memo = bookmark.get('file_memo',{})
        recursive = bookmark.get('option',{}).get('recursive',False)
        apply_filter = bookmark.get('option',{}).get('apply_filter',False)
        filter_word = bookmark.get('option',{}).get('filter_word','')
        filter_exclusion = bookmark.get('option',{}).get('filter_exclusion','')
        filter_ext = bookmark.get('option',{}).get('filter_ext','ma, mb, abc')
        latest = bookmark.get('option',{}).get('latest',False)
        apply_doubleclick = bookmark.get('option',{}).get('apply_doubleclick',False)
        dc_action = bookmark.get('option',{}).get('dc_action',0)

        self.block_filtering(True)
        self.ui.apply_filter.setChecked(apply_filter)
        self.ui.search.setText(filter_word)
        self.ui.exclusion.setText(filter_exclusion)
        self.ui.filter.setCurrentText(filter_ext)
        self.ui.latest.setChecked(latest)
        self.block_filtering(False)

        self.ui.recursive.setChecked(recursive)

        if apply_doubleclick:
            self.dc_actions.button(dc_action).setChecked(True)
            self.set_mouse_cursor()

        if '$PROJ' in dir:
            proj_dir = os.path.dirname(cmds.workspace( q=True ,fullName=True))
            dir = os.path.normpath(dir.replace('$PROJ',proj_dir)).replace('\\','/')

        self.ui.location.setText(dir)

### DIRECTORY_OPERATIONS
    def upper_directory(self):
        current = self.ui.location.text()
        upper = os.path.dirname(current).replace('\\','/')
        self.ui.location.setText(upper)

    def open_filedialog(self):
        current = self.ui.location.text()
        path = QFileDialog.getExistingDirectory(self,'Select Directory',current)
        if path:
            self.ui.location.setText(path.replace('\\','/'))

    def directory_changed(self):
        dir = self.ui.location.text()
        if dir != '' and os.path.isdir(dir):
            self.ui.statusBar.showMessage(dir,3000)
            recursive = self.ui.recursive.isChecked()
            self.all_files = [x.replace('\\','/') for x in glob.glob(f'{dir}/**', recursive=recursive)]
        else:
            self.all_files = []

        self.ui.sort_each_folder.setEnabled(self.ui.recursive.isChecked())

        self.set_file_list()

### FILTER_OPERATIONS
    def filter_linedit_changed(self):
        lineEdit = self.sender()
        text = lineEdit.text()
        object_name = lineEdit.objectName()

        if len(text)>0:
            if self.clear_action[object_name] is None:
                self.clear_action[object_name] = lineEdit.addAction(QIcon('%s/close.png'%icon_dir), QLineEdit.TrailingPosition)
                self.clear_action[object_name].triggered.connect(partial(self.filter_linedit_clear,lineEdit))
        else:
            lineEdit.removeAction(self.clear_action[object_name])
            self.clear_action[object_name] = None

        if not self.block_filter:
            self.set_file_list()

    def filter_linedit_clear(self,lineEdit):
        lineEdit.setText('')

### FILE_LIST_OPERATIONS
    def clear_files(self):
        self.ui.files.clear()
        self.files = []
        self.ui.files.setColumnCount(0)
        self.ui.files.setRowCount(0)

    def file_selected(self):
        row = self.ui.files.currentRow()
        if row == -1:
            return
        file = self.files[row]
        self.ui.statusBar.showMessage(file['Filename'],3000)

        rows = list(set([idx.row() for idx in self.ui.files.selectedIndexes()]))
        if len(rows)==0:
            self.ui.open_file.setEnabled(False)
            self.ui.import_file.setEnabled(False)
            self.ui.reference_file.setEnabled(False)
            self.ui.gpucache_file.setEnabled(False)
            self.ui.open_explorer.setEnabled(False)
        else:
            self.ui.import_file.setEnabled(True)
            self.ui.reference_file.setEnabled(True)
            self.ui.gpucache_file.setEnabled(True)
            self.ui.open_explorer.setEnabled(True)

            if len(rows) == 1:
                self.ui.open_file.setEnabled(True)
            else:
                self.ui.open_file.setEnabled(False)

    def sorted(self):
        section = self.ui.files.horizontalHeader().sortIndicatorSection()
        order = self.ui.files.horizontalHeader().sortIndicatorOrder()
        section_label = self.ui.files.horizontalHeaderItem(section).text()
        self.sort = [section_label, order]

        self.set_file_list()

    def file_doubleclicked(self):
        row = self.ui.files.currentRow()
        if row == -1:
            return
        checked = self.dc_actions.checkedButton()
        mode = checked.objectName().split('_')[1]
        if mode == 'none':
            return
        else:
            self.open_file(mode)

    def open_file(self,mode='open'):
        rows = []
        for idx in self.ui.files.selectedIndexes():
            if idx.row() not in rows:
                rows.append(idx.row())

        if len(rows) == 0:
            return

        repeat = self.ui.repeat_num.value()
        if mode == 'open':
            repeat = 1

        for n in range(repeat):
            for row in rows:
                file_path = self.files[row]['file_path']
                ext = os.path.splitext(file_path)[-1]
                ns = os.path.splitext(os.path.basename(file_path))[0]
                if self.ui.repeat_enable.isChecked():
                    ns += '_%02d'%(n+1)
                if ext == '.ma' or ext == '.mb':        
                    if mode=='open':
                        cmds.file(file_path,open=True,force=True)
                        if self.ui.set_project.isChecked():
                            workspace_file = self.get_nearest_workspace(file_path)
                            if workspace_file is not None:
                                setproj='setProject("'+workspace_file+'");'
                                mel.eval(setproj)
                    elif mode == 'import':
                        cmds.file(file_path,i=True,preserveReferences=True)
                    elif mode == 'reference':
                        cmds.file(file_path,r=True,mergeNamespacesOnClash=0,namespace=ns)
                elif ext == '.abc':
                    if mode=='open':
                        cmds.file(file_path,type='Alembic',open=True,force=True)
                    elif mode == 'import':
                        cmds.AbcImport(file_path)
                    elif mode == 'reference':
                        cmds.file(file_path,r=True,type='Alembic',mergeNamespacesOnClash=0,namespace=ns)
                    elif mode == 'gpucache':
                        gpu_cache = cmds.createNode('gpuCache',name=f'{ns}Shape')
                        transform = cmds.listRelatives(gpu_cache,parent=True)[0]
                        cmds.rename(transform,ns)
                        cmds.setAttr(f'{gpu_cache}.cacheFileName',file_path,type='string')

    def set_file_list(self):
        apply_filter = self.ui.apply_filter.isChecked()
        self.ui.filters.setEnabled(apply_filter)

        self.clear_files()
        if len(self.all_files) == 0:
            return
        
        dir = self.ui.location.text()
        recursive = self.ui.recursive.isChecked()
        filter = self.ui.filter.currentText()
        if filter == '*' or not apply_filter:
            file_filter = None
        else:
            file_filter = ['.'+x.strip() for x in filter.split(',')]

        keyword = self.ui.search.text()
        exclusions = self.ui.exclusion.text().strip().split(' ')

        version_check = {}
        pattern = re.compile('_v[0-9]+')

        i = 0
        for file in self.all_files:
            if os.path.isfile(file):
                if file_filter is None or os.path.splitext(file)[-1] in file_filter:
                    basename = os.path.basename(file)
                    match_filter = True
                    if apply_filter:
                        if keyword != '' and keyword not in basename:
                            match_filter = False
                        
                        for exclusion in exclusions:
                            if exclusion != '' and exclusion  in file:
                                match_filter = False

                    if match_filter:
                        color = self.memo.get(file,{}).get('color','')
                        memo_text = self.memo.get(file,{}).get('text','')
                        m_time = os.path.getmtime(file)
                        size = os.path.getsize(file)
                        n_layer = len(dir.split('/'))
                        parent = '/'.join(os.path.dirname(file).replace('\\','/').split('/')[n_layer:])
                        self.files.append({'file_path':file,'':color,'Filename':basename,'Parent':parent,'Date':m_time,'Size':size,'Memo':memo_text})

                        pos = -1
                        ver = 0
                        m_iter = pattern.finditer(file)
                        for m in m_iter:
                            pos = m.start()
                            ver = int(m.group()[2:])
                        if pos > -1:
                            prefix  = file[:pos]
                            _ver = version_check.get(prefix,[None])[0]
                            if _ver is None or _ver < ver:
                                version_check[prefix] = [ver,i]
                        else:
                            version_check[file] = [-1,i]
                        i+=1

        if apply_filter and self.ui.latest.isChecked():
            use_files = [x[1] for x in version_check.values()]
            self.files = [x for i,x in enumerate(self.files) if i in use_files]

        column_count = 6
        if recursive:
            column_labels = ['','Filename','Parent','Date','Size','Memo']
        else:
            column_labels = ['','Filename','Date','Size','Memo']
            column_count -= 1

        order = (self.sort[1] == Qt.DescendingOrder )
        self.files = sorted(self.files, key=lambda x: x[self.sort[0]],reverse=order)

        if self.ui.sort_each_folder.isEnabled() and self.ui.sort_each_folder.isChecked():
            _parents = {}
            for f in self.files:
                _children = _parents.get(f['Parent'],[])
                _children.append(f)
                _parents[f['Parent']] = _children
            sorted_files = []
            for _children in _parents.values():
                sorted_files += _children
            self.files = sorted_files

        self.ui.files.setColumnCount(column_count)
        self.ui.files.setHorizontalHeaderLabels(column_labels)

        self.ui.files.setRowCount(len(self.files))

        for row,file in enumerate(self.files):
            color = file['']
            bg_brush = None
            if color != '' and color != 'none':
                bg_brush = QBrush(QColor(color))

            color_item = QTableWidgetItem(f'')
            if bg_brush:
                color_item.setBackground(bg_brush)
            self.ui.files.setItem( row, 0, color_item)

            ext = os.path.splitext(file['file_path'])[-1][1:]
            icon = self.file_icon.get(ext,self.file_icon_none)
            name_item = QTableWidgetItem(icon,f'  {file["Filename"]}  ')

            self.ui.files.setItem( row, 1, name_item)
            
            column = 2
            if recursive:
                parent_item = QTableWidgetItem(f'  {file["Parent"]}  ')
                self.ui.files.setItem( row, 2 , parent_item)
                column += 1
            
            dt = datetime.datetime.fromtimestamp(file['Date'])
            datetime_str = dt.strftime('%Y/%m/%d %H:%M:%S')
            datetime_item = QTableWidgetItem(f'  {datetime_str}  ')
            self.ui.files.setItem( row, column, datetime_item )
            column += 1

            size = convert_size(file['Size'])
            self.ui.files.setItem( row, column, QTableWidgetItem(f'  {size}  '))
            column += 1

            memo_text_item = QTableWidgetItem(f'  {file["Memo"]}  ')
            self.ui.files.setItem( row, column, memo_text_item )

        self.ui.files.horizontalHeader().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
        if self.sort[0] in column_labels:
            sort_indicator = column_labels.index(self.sort[0])
            self.ui.files.horizontalHeader().setSortIndicator(sort_indicator,self.sort[1])

### OPEN EXPLORER
    def open_explorer_from_bookmark(self,index):
        if index.row() == -1:
            self.clear_files()
            return
        
        bookmark = self.bookmarks[self.cur_group][index.row()]
        dir = bookmark.get('directory',None)
        if dir:
            subprocess.Popen(['explorer',os.path.normpath(dir)],shell=True)

    def open_explorer_from_file(self):
        row = self.ui.files.currentRow()
        if row == -1:
            return
        file = self.files[row]
        directory = os.path.dirname(file['file_path'])
        subprocess.Popen(['explorer',os.path.normpath(directory)],shell=True)

### COMMON FUNCS
    def get_nearest_workspace(self,filename):
        _path = filename.replace('\\','/')
        splited = _path.split('/')
        for i in range(len(splited)):
            ws = '/'.join(splited[:-i]) + '/workspace.mel'
            if os.path.isfile(ws):
                return os.path.dirname(ws).replace('\\','/')
        return None        

### OTHER
    def open_reference_editor(self):
        cmds.ReferenceEditor()


def main():
    app=QApplication.instance()
    ui=btkFileManager()

    ui.show()

if __name__ == '__main__':
    main()    