# -*- coding: utf-8 -*-
######################################################
#
#  Slide Vari Setter
#   Copyright (c) 2024 btk
#   mail@btk.tokyo.jp
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to use
# the Software with or without modification, subject to the following conditions:
# 
# 1. The code may be modified and used freely.
# 2. If you modify the code, please notify the original copyright holder
#    (mail@btk.tokyo.jp) of the modifications made, if possible.
# 3. Redistribution of the original or modified code is prohibited.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# 本ソフトウェアおよび関連文書ファイル（以下「ソフトウェア」）のコピーを
# 入手したすべての人に対し、以下の条件に従うことを条件に、使用および改変の
# 許可を無償で付与します。
# 
# 1. 本コードは自由に改変および使用してかまいません。
# 2. 可能であれば、改変した場合は改変の内容を元の著作権者（mail@btk.tokyo.jp）に連絡してください。
# 3. 元のコードおよび改変されたコードの再配布は禁止します。
# 
# 本ソフトウェアは「現状のまま」提供されており、商品性、特定の目的への適合性、
# および権利非侵害についての保証を含め、明示的または黙示的な保証はありません。
# 著者または著作権者は、契約行為、不法行為、またはその他の行為に関係なく、
# 本ソフトウェアまたはその使用またはその他の取り扱いから生じるいかなる
# 請求、損害、その他の責任についても責任を負いません。
#
######################################################
try:
    from PySide2.QtCore import (QPoint, Qt, Signal)
    from PySide2.QtGui import (QIntValidator, QDoubleValidator, QCursor)
    from PySide2.QtWidgets import (QLineEdit, QMenu, QSpinBox, QDoubleSpinBox, QSizePolicy)
    MID_BUTTON = Qt.MidButton
except:
    from PySide6.QtCore import (QPoint, Qt, Signal)
    from PySide6.QtGui import (QIntValidator, QDoubleValidator, QCursor)
    from PySide6.QtWidgets import (QLineEdit, QMenu, QSpinBox, QDoubleSpinBox, QSizePolicy)
    MID_BUTTON = Qt.MiddleButton

class BTKMenuSlideVariSetter(QMenu):
    value_changed = Signal(object)
    closed = Signal()
    
    def __init__(self, parent=None):
        super(BTKMenuSlideVariSetter, self).__init__(parent)

        self.setSizePolicy(QSizePolicy.Minimum,QSizePolicy.Minimum)
        self.setStyleSheet('QMenu::item {padding: 2px 5px 2px 2px;}')

        self.current_act = None
        self.value_change_mode = False

    def showEvent(self,event):
        for _a in self.actions():
            _a.setEnabled(False)
        self.min_x = self.rect().x()
        self.max_x = self.rect().right()


    def mouseMoveEvent(self,event):
        pos = event.localPos()

        act_point = QPoint(1,pos.y())

        for _a in self.actions():
            _a.setEnabled(False)

        if self.current_act is not None:
            x = pos.x()
            if x < self.min_x:
                self.value_change_mode = True
                ret_val = int((x-self.min_x)/10)
            elif x > self.max_x:
                self.value_change_mode = True
                ret_val = int((x-self.max_x)/10)
            else:
                self.value_change_mode = False
                ret_val = 0
            value = float(self.current_act.text())*float(ret_val)
            self.value_changed.emit(value)

        if not self.value_change_mode:
            act= self.actionAt(act_point)
            if act is not None:
                self.current_act = act
                act.setEnabled(True)
    
        if self.current_act:
            self.current_act.setEnabled(True)

    def mouseReleaseEvent(self,event):
        self.close()
        self.closed.emit()

class BTKLineEditSlideVari(QLineEdit):
    def __init__(self, value='', parent=None):
        super(BTKLineEditSlideVari, self).__init__(value, parent)

        self.cursor = QCursor()
        self.cur_pos = None
        self.value_type = None
        self.current_value = None
        self.min_digit = None
        self.validator = None
        self.menu = None

    def set_validator(self,value_type='int',top=None,bottom=None):
        self.value_type = value_type
        if value_type=='int':
            self.validator = QIntValidator()
        elif value_type=='float':
            self.validator = QDoubleValidator()

        if top is not None:
            self.validator.setTop(top)
        if bottom is not None:
            self.validator.setTop(bottom)

        self.setValidator(self.validator)
        if self.text() == '':
            self.setText('0')

    def set_validator_range(self,top=None,bottom=None):
        if top is not None:
            self.validator.setTop(top)
        if bottom is not None:
            self.validator.setTop(bottom)

    def menu_setup(self,max_digit=3,min_digit=3):
        if self.menu is not None:
            self.menu.deleteLater()
        self.menu = BTKMenuSlideVariSetter(self)
        self.menu.value_changed.connect(self.value_changed)
        self.menu.closed.connect(self.menu_closed)

        if self.value_type == 'int':
            min_digit = 0
        elif self.value_type == 'float':
            self.validator.setDecimals(min_digit)

        self.min_digit = min_digit
        for d in range(-min_digit,max_digit):
            v = pow(10,d)
            self.menu.addAction(f'{v}')

    def mousePressEvent(self,event):
        if event.button() == MID_BUTTON:
            text = self.text()
            if text == '':
                text = '0'
            if self.value_type=='int':
                self.current_value = int(text)
            elif self.value_type=='float':
                self.current_value = float(text)

            self.cur_pos = event.globalPos()
            point = QPoint(self.cur_pos.x(),self.cur_pos.y())
            self.menu.exec_(point)
        else:
            super(BTKLineEditSlideVari, self).mousePressEvent(event)

    def value_changed(self,value):
        if self.value_type == 'int':
            _value = int(self.current_value + value)
        elif self.value_type == 'float':
            _value = round( float(self.current_value + value) , self.min_digit)
        self.setText(f'{_value}')

    def menu_closed(self):
        self.cursor.setPos(self.cur_pos)

class BTKSpinBoxEditSlideVari(QSpinBox):
    def __init__(self, parent=None):
        super(BTKSpinBoxEditSlideVari, self).__init__(parent)

        self.cursor = QCursor()
        self.cur_pos = None
        self.current_value = None
        self.line_edit = None
        self.menu = None

    def menu_setup(self,max_digit=3):
        if self.menu is not None:
            self.menu.deleteLater()
        self.menu = BTKMenuSlideVariSetter(self)
        self.menu.value_changed.connect(self.value_changed)
        self.menu.closed.connect(self.menu_closed)

        for d in range(max_digit):
            v = pow(10,d)
            self.menu.addAction(f'{v}')

        if self.line_edit is not None:
            self.line_edit.deleteLater()
        self.line_edit = BTKLineEditSlideVari(self)
        self.line_edit.set_validator('int',top=self.maximum(),bottom=self.minimum())
        self.line_edit.menu_setup(max_digit=max_digit,min_digit=0)
        self.setLineEdit(self.line_edit)

    def mousePressEvent(self,event):
        if event.button() == MID_BUTTON:
            self.current_value = self.value()

            self.cur_pos = event.globalPos()
            point = QPoint(self.cur_pos.x(),self.cur_pos.y())
            self.menu.exec_(point)
        else:
            super(BTKSpinBoxEditSlideVari, self).mousePressEvent(event)

    def value_changed(self,value):
        self.setValue(self.current_value + value)

    def menu_closed(self):
        self.cursor.setPos(self.cur_pos)

    def setMaximum(self,value):
        super(BTKSpinBoxEditSlideVari, self).setMaximum(value)
        if self.line_edit is None:
            self.menu_setup()
        self.line_edit.set_validator_range(top=value)

    def setMinimum(self,value):
        super(BTKSpinBoxEditSlideVari, self).setMinimum(value)
        if self.line_edit is None:
            self.menu_setup()
        self.line_edit.set_validator_range(bottom=value)

class BTKDoubleSpinBoxEditSlideVari(QDoubleSpinBox):
    def __init__(self, parent=None):
        super(BTKDoubleSpinBoxEditSlideVari, self).__init__(parent)

        self.cursor = QCursor()
        self.cur_pos = None
        self.value_type = None
        self.min_digit = None
        self.line_edit = None
        self.menu = None

    def menu_setup(self,max_digit=3,min_digit=3):
        if self.menu is not None:
            self.menu.deleteLater()
        self.menu = BTKMenuSlideVariSetter(self)
        self.menu.value_changed.connect(self.value_changed)
        self.menu.closed.connect(self.menu_closed)

        self.min_digit = min_digit
        for d in range(-min_digit,max_digit):
            v = pow(10,d)
            self.menu.addAction(f'{v}')

        self.setDecimals(min_digit)

        if self.line_edit is not None:
            self.line_edit.deleteLater()
        self.line_edit = BTKLineEditSlideVari(self)
        self.line_edit.set_validator('float',top=self.maximum(),bottom=self.minimum())
        self.line_edit.menu_setup(max_digit=max_digit,min_digit=min_digit)
        self.setLineEdit(self.line_edit)

    def mousePressEvent(self,event):
        if event.button() == MID_BUTTON:
            self.current_value = self.value()

            self.cur_pos = event.globalPos()
            point = QPoint(self.cur_pos.x(),self.cur_pos.y())
            self.menu.exec_(point)
        else:
            super(BTKDoubleSpinBoxEditSlideVari, self).mousePressEvent(event)

    def value_changed(self,value):
        self.setValue(self.current_value + value)

    def menu_closed(self):
        self.cursor.setPos(self.cur_pos)

    def setMaximum(self,value):
        super(BTKSpinBoxEditSlideVari, self).setMaximum(value)
        if self.line_edit is None:
            self.menu_setup()
        self.line_edit.set_validator_range(top=value)

    def setMinimum(self,value):
        super(BTKSpinBoxEditSlideVari, self).setMinimum(value)
        if self.line_edit is None:
            self.menu_setup()
        self.line_edit.set_validator_range(bottom=value)

